import React, { forwardRef, useEffect, useImperativeHandle, useState, } from 'react';
import { ActivityIndicator, Dimensions, FlatList, RefreshControl, Text, View, } from 'react-native';
import { gDp } from 'th_comm.rn';
/**
 *
 * L: is the list type.
 *
 * D: is the data type.
 */
function LoadList(props, ref) {
    var windowWidth = Dimensions.get('window').width; // 屏幕宽高
    var _a = props.showInitLoading, showInitLoading = _a === void 0 ? true : _a, _b = props.canRefresh, canRefresh = _b === void 0 ? false : _b;
    var listDataHandler = props.listDataHandler, _c = props.size, size = _c === void 0 ? 10 : _c, _d = props.emptyConfig, emptyConfig = _d === void 0 ? { hint: '数据为空' } : _d, _e = props.dividerHeight, dividerHeight = _e === void 0 ? 0 : _e;
    var _f = useState(undefined), data = _f[0], setData = _f[1];
    // how many page that got.
    var _g = useState(0), page = _g[0], setPage = _g[1];
    // 是否正在数据，防重复加载
    var _h = useState(false), isLoadingData = _h[0], setIsLoadingData = _h[1];
    var _j = useState(false), showError = _j[0], setShowError = _j[1];
    var _k = useState(false), showLoading = _k[0], setShowLoading = _k[1];
    var _l = useState(true), isFirstTimeLoadData = _l[0], setIsFirstTimeLoadData = _l[1];
    var _m = useState(false), noMoreData = _m[0], setNoMoreData = _m[1];
    var dataIsEmpty = data && data.length === 0;
    var buildEmpty = function () {
        if (emptyConfig.builder) {
            return <emptyConfig.builder />;
        }
        else {
            return <Text>数据为空</Text>;
        }
    };
    var loadData = function (index, pShowLoading) {
        if (index === void 0) { index = 1; }
        if (pShowLoading === void 0) { pShowLoading = showInitLoading; }
        console.log('refresh called, isLoading: ', isLoadingData);
        if (isLoadingData) {
            return;
        }
        setIsLoadingData(true);
        if (index === 1 && pShowLoading) {
            console.log('setShowLoading(true) called');
            setShowLoading(true);
        }
        props
            .dataLoader({ index: index, size: size })
            .then(function (listData) {
            var _a;
            if (!listDataHandler.isOk(listData)) {
                setShowError(true);
            }
            else {
                setShowError(false);
                setIsFirstTimeLoadData(false);
                setNoMoreData(!listDataHandler.hasMore(listData));
                var newList = [];
                if (index !== 1) {
                    newList.push.apply(newList, (data !== null && data !== void 0 ? data : []));
                }
                newList.push.apply(newList, ((_a = listDataHandler.getData(listData)) !== null && _a !== void 0 ? _a : []));
                setData(newList);
                setPage(index);
            }
            setIsLoadingData(false);
            setShowLoading(false);
        })
            .catch(function (err) {
            console.error(err);
            setShowLoading(false);
            setShowError(true);
            setIsLoadingData(false);
        });
    };
    useImperativeHandle(ref, function () { return ({
        refresh: function (pShowLoading) {
            loadData(1, pShowLoading);
        },
        changeData: function (changer) {
            setData(changer(data || []));
        },
    }); });
    var refresh = function () { return loadData(1); };
    // 这里在首页的时候为什么会调用两次？
    // 调用两次，说明控件被回收，又被创建了。
    useEffect(function () {
        refresh();
    }, []);
    if (showError) {
        return <Text>出错了。请稍后刷新重试</Text>;
    }
    if (dataIsEmpty) {
        return buildEmpty();
    }
    if (showLoading && isFirstTimeLoadData) {
        // noinspection JSSuspiciousNameCombination
        return (<View style={{
                alignItems: 'center',
                justifyContent: 'center',
                width: '100%',
                height: windowWidth,
            }}>
              <ActivityIndicator size={'large'} color={'green'}/>
          </View>);
    }
    var handleLoadMore = function () {
        if (noMoreData) {
            return;
        }
        loadData(page + 1);
    };
    var _LoadMoreIndicatorView = function () {
        if (noMoreData) {
            return null;
        }
        return (<View style={{
                height: gDp(100),
                alignItems: 'center',
                justifyContent: 'center',
            }}>
              <ActivityIndicator size={'small'}/>
          </View>);
    };
    return (<FlatList showsVerticalScrollIndicator={false} showsHorizontalScrollIndicator={false} {...props.flatListProps} data={data !== null && data !== void 0 ? data : []} renderItem={props.renderItem} refreshControl={(canRefresh && (<RefreshControl onRefresh={refresh} refreshing={false}/>)) ||
            undefined} ListFooterComponent={_LoadMoreIndicatorView} ItemSeparatorComponent={function () {
            return (<View style={{
                    height: dividerHeight,
                }}/>);
        }} onEndReached={handleLoadMore} onEndReachedThreshold={0.4}/>);
}
export default forwardRef(LoadList);
//# sourceMappingURL=LoadList.js.map